@@ -285,8 +285,8 @@ static const GtkActionEntry UI_ENTRIES[] = {
 	  N_("Delete selected items"), G_CALLBACK (on_object_delete) },
 	{ "properties-object", GTK_STOCK_PROPERTIES, NULL, NULL,
 	  N_("Show the properties of this item"), G_CALLBACK (on_properties_object) },
-	{ "properties-place", GTK_STOCK_PROPERTIES, NULL, NULL,
-	  N_("Show the properties of this place"), G_CALLBACK (on_properties_place) },
+	{ "properties-keyring", GTK_STOCK_PROPERTIES, NULL, NULL,
+	  N_("Show the properties of this keyring"), G_CALLBACK (on_properties_place) },
 	{ "app-preferences", GTK_STOCK_PREFERENCES, N_("Prefere_nces"), NULL,
 	  N_("Change preferences for this program"), G_CALLBACK (on_app_preferences) },
 	{ "view-menu", NULL, N_("_View") },
