@@ -31,19 +31,9 @@
 #include <gtk/gtk.h>
 #include <time.h>
 
-#include "seahorse-context.h"
-
-#ifdef WITH_SHARING
-#include <avahi-client/client.h>
-const AvahiPoll* seahorse_util_dns_sd_get_poll (void);
-#endif
-
-struct _SeahorseObject;
-
 typedef guint64 SeahorseVersion;
 
-#define SEAHORSE_EXT_ASC ".asc"
-#define SEAHORSE_EXT_GPG ".gpg"
+#define SEAHORSE_BAD_FILENAME_CHARS  "/\\<>|:?;"
 
 gchar*      seahorse_util_get_date_string           (const time_t time);
 gchar*      seahorse_util_get_display_date_string   (const time_t time);
@@ -53,18 +43,15 @@ gchar*      seahorse_util_get_display_date_string   (const time_t time);
 
 GQuark      seahorse_util_error_domain          (void);
 
-void        seahorse_util_show_error            (GtkWidget          *parent,
-                                                 const gchar        *heading,
-                                                 const gchar        *message);
+void        seahorse_util_show_error            (gpointer parent,
+                                                 const gchar *heading,
+                                                 const gchar *message);
 
 void        seahorse_util_handle_error          (GError **error,
                                                  gpointer parent,
                                                  const gchar* description,
                                                  ...);
 
-gboolean    seahorse_util_prompt_delete         (const gchar *text,
-                                                 GtkWidget *parent);
-
 guchar*     seahorse_util_read_to_memory        (GInputStream *     input,
                                                  guint              *len);
 
@@ -78,49 +65,17 @@ gboolean    seahorse_util_print_fd          (int fd,
 
 gboolean    seahorse_util_printf_fd         (int fd, 
                                              const char* data, ...);
-                             
-gchar*      seahorse_util_filename_for_objects (GList *objects);
-                                             
-gboolean    seahorse_util_uri_exists        (const gchar* uri);
 
-gchar*      seahorse_util_uri_unique        (const gchar* uri);
-
-const gchar* seahorse_util_uri_get_last     (const gchar* uri);
-
-GQuark      seahorse_util_detect_mime_type   (const gchar *mime);
-
-GQuark      seahorse_util_detect_data_type   (const gchar *data,
-                                              guint length);
-
-GQuark      seahorse_util_detect_file_type   (const gchar *uri);
+GFile *     seahorse_util_file_increment_unique     (GFile *file,
+                                                     guint *state);
 
 gboolean    seahorse_util_write_file_private            (const gchar* filename,
                                                          const gchar* contents,
                                                          GError **err);
 
-GtkDialog*  seahorse_util_chooser_open_new              (const gchar *title,
-                                                         GtkWindow *parent);
+GList *     seahorse_util_objects_sort_by_place         (GList *objects);
 
-GtkDialog*  seahorse_util_chooser_save_new              (const gchar *title,
-                                                         GtkWindow *parent);
-
-void        seahorse_util_chooser_show_key_files        (GtkDialog *dialog);
-
-void        seahorse_util_chooser_show_archive_files    (GtkDialog *dialog);
-
-void        seahorse_util_chooser_set_filename_full     (GtkDialog *dialog, 
-                                                         GList *objects);
-
-void        seahorse_util_chooser_set_filename          (GtkDialog *dialog, 
-                                                         struct _SeahorseObject *object);
-
-gchar*      seahorse_util_chooser_open_prompt           (GtkDialog *dialog);
-
-gchar*      seahorse_util_chooser_save_prompt           (GtkDialog *dialog);
-
-GList*      seahorse_util_objects_sort       (GList *objects);
-
-GList*      seahorse_util_objects_splice     (GList *objects);
+GList *     seahorse_util_objects_splice_by_place       (GList *objects);
 
 gboolean    seahorse_util_string_equals       (const gchar *s1, const gchar *s2);
 
